$nuvem = Get-SCCloud -VMMServer "{{ servidor_vmm }}" -Name "{{ nuvem }}"
if (!$nuvem ) {
  Write-Host "Nuvem (Cloud) '{{ nuvem }}' não encontrada."
}

{% for imagem in imagens %}
$imagem = Get-SCVMTemplate -VMMServer "{{ servidor_vmm }}" | Where {$_.Name -eq "{{ imagem }}"}
if (!$imagem ) {
  Write-Host "Imagem (Template) '{{ imagem }}' não encontrada."
}
{% endfor %}

$qtde_minima_regioes = {{ qtde_minima_regioes }}
$qtde_regioes = (Get-SCVMHost -VMMServer "{{ servidor_vmm }}" | Where-Object {$_.OverallState -eq "OK"} | Measure-Object).Count
if ($qtde_minima_regioes -gt $qtde_regioes) {
  Write-Host "Número de regiões incompatível. Disponíveis: $qtde_regioes. Quantidade necessária: $qtde_minima_regioes."
}

{% for rede in redes %}
$rede = Get-SCVMNetwork -VMMServer "{{ servidor_vmm }}" -Name "{{ rede }}"
if (!$rede ) {
  Write-Host "Rede '{{ rede }}' não encontrada."
}
{% endfor %} 

if ($nuvem ) {
  $custom_agrupamento = Get-SCCustomProperty -VMMServer "{{ servidor_vmm }}" -Name "{{ campo_agrupamento }}"
  $vms_nuvem = Get-SCVirtualMachine -VMMServer "{{ servidor_vmm }}" -Cloud $nuvem
  $nomes_vm_desejados = @({{ lista_nome_vms_str }})
  foreach($vm in $vms_nuvem) {
    $agrupamento = Get-SCCustomPropertyValue -InputObject $vm -CustomProperty $custom_agrupamento
    if ($agrupamento.Value -ne "{{ agrupamento }}" -and $nomes_vm_desejados -Contains $vm.Name) {
      Write-Output "O nome '$vm' já está sendo usado por uma VM na nuvem '$nuvem'. Por favor, escolha outro nome."
    }
  }
}
